@@ -749,14 +749,15 @@ class StringWithSpaceOnRight(String):
 
 class Regexp(Value):
     """Value must be a valid regular expression."""
-    __slots__ = ('sr', 'value', '__parent')
     errormsg = _('Value must be a valid regular expression, not %r.')
-    def __init__(self, *args, **kwargs):
-        kwargs['setDefault'] = False
-        self.sr = ''
-        self.value = None
-        self.__parent = super(Regexp, self)
-        self.__parent.__init__(*args, **kwargs)
+
+    def __init__(self, default, *args, **kwargs):
+        # We're not supposed to do convertions here, BUT this is needed
+        # when the value is set programmatically because the value
+        # plugins set (a string) is not the same as the one they get
+        # (a compiled pattern object)
+        default = self._convertFromString(default)
+        super().__init__(default, *args, **kwargs)
 
     def error(self, e):
         s = 'Value must be a regexp of the form m/.../ or /.../. %s' % e
@@ -764,29 +765,48 @@ class Regexp(Value):
         e.value = self
         raise e
 
+    def _convertFromString(self, s):
+        if s:
+            # We need to preserve the original string, as it's shown in
+            # the user interface and the config file.
+            # It might be tempting to set the original string as an
+            # attribute, but doing so would result in inconsistent states
+            # for childs of this variable, should they be reset, or the
+            # value of there parent change.
+            return (s, utils.str.perlReToPythonRe(s))
+        else:
+            return None
+
     def set(self, s):
         try:
-            if s:
-                self.setValue(utils.str.perlReToPythonRe(s), sr=s)
-            else:
-                self.setValue(None)
+            v = self._convertFromString(s)
         except ValueError as e:
             self.error(e)
-
-    def setValue(self, v, sr=None):
-        if v is None:
-            self.sr = ''
-            self.__parent.setValue(None)
-        elif sr is not None:
-            self.sr = sr
-            self.__parent.setValue(v)
         else:
-            raise InvalidRegistryValue('Can\'t setValue a regexp, there would be an inconsistency '\
-                  'between the regexp and the recorded string value.')
+            super().set(v)
+
+    def setValue(self, v):
+        """Don't call this function directly from plugins, it is subject
+        to change without notice."""
+        if v is not None and (not isinstance(v, tuple) or len(v) != 2):
+            raise InvalidRegistryValue(
+                'Can\'t setValue a regexp, there would be an inconsistency '
+                'between the regexp and the recorded string value. '
+                'Use .set() instead.')
+
+        super().setValue(v)
+
+    def __call__(self):
+        if self.value is None:
+            return None
+        else:
+            return self.value[1]
 
     def __str__(self):
-        self() # Gotta update if we've been reloaded.
-        return self.sr
+        if self.value is None:
+            return ''
+        else:
+            return self.value[0]
 
 class SeparatedListOf(Value):
     __slots__ = ()
